<?php

declare(strict_types=1);

namespace Drupal\google_v3_glossary\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystem;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\google_v3_glossary\Services\GoogleV3GlossaryFileManagerService;
use Drupal\google_v3_glossary\Traits\GoogleV3TranslatorCSVTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a dynamic language glossary form.
 */
final class DynamicLanguageGlossaryForm extends FormBase {
  use GoogleV3TranslatorCSVTrait;

  /**
   * Service for handling the file authentication, upload and download.
   *
   * @var \Drupal\google_v3_glossary\Services\GoogleV3GlossaryFileManagerService
   */
  protected readonly GoogleV3GlossaryFileManagerService $fileManager;

  /**
   * Drupal file system service.
   *
   * @var \Drupal\Core\File\FileSystem|mixed
   */
  protected FileSystem $fileSystem;

  /**
   * The language code for this form instance.
   *
   * @var string
   */
  protected string $langCode = '';

  /**
   * Config factory service.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private $config;

  /**
   * Private file path to save the glossary file in Drupal.
   *
   * @var string
   */
  protected string $destinationPath;

  /**
   * Private file path to save the glossary file in Drupal.
   *
   * @var string
   */
  protected string $path;

  /**
   * {@inheritdoc}
   */
  public function __construct(FileSystem $fileSystem, GoogleV3GlossaryFileManagerService $fileManager, ConfigFactoryInterface $configFactory, ?RouteMatchInterface $route_match = NULL) {
    $this->fileSystem = $fileSystem;
    $this->fileManager = $fileManager;

    $this->config = $configFactory->get('google_v3_glossary.settings');
    // Get the language code from the route.
    $this->langCode = $route_match->getParameter('langCode');
    $this->path = $this->config->get('file_path_' . $this->langCode);
    $this->validateCreateDestinationPath($this->path);
    $this->destinationPath = $this->fileSystem->realpath("private://{$this->path}");
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_system'),
      $container->get('google_v3_glossary.file.manager'),
      $container->get('config.factory'),
      $container->get('current_route_match'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'google_v3_glossary_glossary';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['translations'] = [
      '#type' => 'textarea',
      '#title' => $this->t('@language language glossary', ['@language' => strtoupper($this->langCode)]),
      '#description' => $this->t('This is a CSV style field. One comma separated word pair per line. e.g. "Hello,Hola"'),
      '#required' => FALSE,
      '#rows' => 35,
      '#default_value' => $this->loadData("private://{$this->path}", $this->langCode, $this->destinationPath) ?? '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['google_v3_glossary_glossary.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // Validate that the textarea is valid CSV.
    $validated = $this->validateCsv($form_state->getValue('translations'));

    if (!$validated['valid']) {
      $form_state->setErrorByName('translations', $validated['message']);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    // Get and save CSV values from the form field.
    $translations = $form_state->getValue('translations');
    $this->saveCsvFile($translations, $this->destinationPath);

    // Google V3 Glossary.
    $this->fileManager->uploadGlossaryToGoogle($this->destinationPath, $this->langCode);

    $this->messenger()->addStatus($this->t('Saved.'));
  }

  /**
   * Load data from CSV file.
   *
   * @param string $fileName
   *   The name of the file to load.
   * @param string $langCode
   *   The language code for the file.
   * @param string $destinationPath
   *   The path to the destination where the file will be saved.
   */
  protected function loadData(string $fileName, string $langCode, string $destinationPath): string {
    try {
      // Google V3 Glossary.
      $this->fileManager->downloadGlossaryFromGoogle($destinationPath, $langCode);
    }
    catch (\Exception $e) {
      $this->logger('google_v3_glossary')->error($e->getMessage());
    }

    $data = $this->loadCsvFile($fileName);

    if ($data) {
      return $this->convertArrayToCSVString($data);
    }

    return '';
  }

}
